<?php
/**
 * @file
 * Contains \Drupal\role_notices\Form\RoleNoticesSettingsForm.
 */

namespace Drupal\role_notices\Form;

use Drupal\Core\Form\FormBase;
use Drupal\role_notices\NoticesManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Simple Settings form.
 *
 * This settings form does not save it's settings as config(they need to be
 * updated on the site) so it does not extend Drupal\Core\Form\ConfigFormBase;
 */
class RoleNoticesSettingsForm extends FormBase {

  /**
   * The state service.
   *
   * @var \Drupal\role_notices\NoticesManager
   */
  protected $noticesManager;

  /**
   * Class constructor.
   *
   * @param \Drupal\role_notices\NoticesManager $notices_manager
   *   The notices manager for getting and setting notices.
   */
  public function __construct(NoticesManager $notices_manager) {
    $this->noticesManager = $notices_manager;
  }

  /**
   * Create function to provide our necessary service.
   *
   * This works because we are extending FormBase which implements
   * ContainerInjectionInterface.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container that lets us get our service.
   *
   * @link https://www.drupal.org/node/2133171 For more on dependency injection
   */
  public static function create(ContainerInterface $container) {
    return new static(
    /*
     *  Load the service required to construct this class.
     *  Notice that this is the name we use to define our service in role_notices.services.yml
     */
      $container->get('role_notices.notice_manager')
    );
  }

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormID() {
    return 'role_notices_setting_form';
  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param FormStateInterface $form_state
   *   An associative array containing the current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $role_names = user_role_names(TRUE);
    /*
     * Using an empty container with #tree property
     *  will make our values come back as array.
     */
    $form['role_notices_notices'] = array(
      '#tree' => TRUE,
    );
    $notices = $this->noticesManager->getAllNotices();
    // Create 1 text area for each role.
    foreach ($role_names as $role_id => $role_name) {
      $form['role_notices_notices'][$role_id] = array(
        '#type' => 'textarea',
        '#title' => $role_name,
        '#description' => $this->t('Add a notice for the !role role', array('!role' => "<strong>$role_name</strong>")),
        '#default_value' => isset($notices[$role_id]) ? $notices[$role_id] : '',
      );
    }
    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Save Text settings'),
    );
    return $form;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param FormStateInterface $form_state
   *   An associative array containing the current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
    $this->noticesManager->setAllNotices($form_values['role_notices_notices']);
    drupal_set_message($this->t('The notices have been saved.'));
  }

}
