<?php
/**
 * @file
 * Contains \Drupal\role_notices\NoticesManager.
 */

namespace Drupal\role_notices;

use Drupal;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Cache\Cache;

/**
 * Simple Notices Manager.
 *
 * This class is set as service in role_notices.services.yml
 * It will need to be provided to classes via Dependency injection
 *
 * Having a separate class for setting and getting notices means it will easier
 * to change this later on.
 *
 * @see \Drupal\role_notices\Form\RoleNoticesSettingsForm::create()
 */
class NoticesManager {

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * Creates a new Notices Manager.
   *
   * @param \Drupal\Core\Session\AccountInterface $user
   *   The current user.
   * @param Drupal\Core\State\StateInterface $state
   *   Used to get and save notices from site state.
   */
  public function __construct(AccountInterface $user, StateInterface $state) {
    $this->user = $user;
    $this->state = $state;
  }

  /**
   * Utility function to get the notices of the current user.
   *
   * @return array
   *   An array of notices for the current user. keyed by role id(rid)
   */
  public function getUserNotices() {
    $all_notices = $this->state->get('role_notices_notices');
    $user_notices = array_intersect_key($all_notices, array_flip($this->user->getRoles()));
    // @todo Pass in module handler with Dependency Injection pattern?
    \Drupal::moduleHandler()->alter('role_notices', $user_notices);
    return $user_notices;
  }

  /**
   * Get notices for all roles.
   *
   * @return array
   *   Any associative array of notices.
   *    - keys are machine names of roles
   *    - values are notices strings
   */
  public function getAllNotices() {
    return $this->state->get('role_notices_notices');
  }

  /**
   * Set notices for all roles.
   *
   * The notices are saved using Drupal::state()->set().
   * We don't want this setting to be exportable.
   * If we wanted it to be exportable we would use Drupal::config
   *
   * @param array $notices
   *   Any associative array of notices.
   *    - keys are machine names of roles.
   *    - values are notices strings.
   */
  public function setAllNotices(array $notices) {
    // Save the updated keys before saving the new notices.
    $updated_keys = $this->getUpdatedNotices($notices);
    $this->state->set('role_notices_notices', $notices);
    /*
     * Delete render tags for updated Notices.
     * This ensures that if a user has a role that with updated notice they will
     * see the new value.
     * It also ensures if they have a role that was deleted they won't see the
     * old notice.
     */
    if ($updated_keys) {
      Cache::invalidateTags($this->getRenderTags($updated_keys));
    }

    // @todo Also delete tags for notices when the Roles are deleted.
  }

  /**
   * Return the rids for any Notices that have been updated.
   *
   * This all includes new roles and roles that were deleted.
   *
   * @param array $new_notices
   *   Notices that were just submitted to be saved.
   *   The haven't been saved yet.
   *
   * @return array
   *   Role Ids that have been updated or no longer exist.
   */
  protected function getUpdatedNotices(array $new_notices) {
    $current_notices = $this->getAllNotices();
    $diff = array_diff_assoc($current_notices, $new_notices)
      + array_diff_assoc($new_notices, $current_notices);
    return array_keys($diff);
  }

  /**
   * Get the render tags for given rids.
   *
   * @param array $rids
   *   Role Ids.
   *
   * @return array
   *   Render tags in the format: 'role_notices:[ROLE_ID]'.
   */
  public function getRenderTags(array $rids) {
    $tags = array();
    if ($rids) {
      foreach ($rids as $rid) {
        $tags[] = 'role_notices:' . $rid;
      }
    }
    return $tags;
  }

}
